////////////////////////////////////////////////////////////////////////////
//
// In-Circuit Conduction Tester.
//
// Copyright by rubidesu 2019.
// http://www.gatelink.co.jp/hw/
//
// dł삷PIC12LF1822gp.
// RpCXC8.
// >xc8 --chip=12lf1822 cc_pic.c
//
// 2019.11.07 V1.00.
//
////////////////////////////////////////////////////////////////////////////

#include <xc.h>

// CONFIG1
#pragma config FOSC = INTOSC
#pragma config WDTE = OFF
#pragma config PWRTE = ON
#pragma config MCLRE = OFF
#pragma config CP = OFF
#pragma config CPD = OFF
#pragma config BOREN = OFF
#pragma config CLKOUTEN = OFF
#pragma config IESO = OFF
#pragma config FCMEN = OFF

// CONFIG2
#pragma config WRT = OFF
#pragma config PLLEN = OFF
#pragma config STVREN = OFF
#pragma config BORV = LO
#pragma config LVP = OFF

#define _XTAL_FREQ 125000

//^`.
typedef unsigned char	BYTE;
typedef unsigned bit	BIT;
typedef unsigned short	WORD;
typedef unsigned long	DWORD;

//High/Low̒`.
#define HIGH 1
#define LOW  0

#define BIAS_PORT	LATAbits.LATA5
#define LED_PORT	LATAbits.LATA0
#define SW_PORT		PORTAbits.RA3

#define SW_NONE		0
#define SW_SHORT	1
#define SW_LONG		2

////////////////////////////////////////////////////////////////////////////
WORD get_adval(void)
{
	WORD wADVal;

	BIAS_PORT = HIGH;
	__delay_us(10);

	ADCON0bits.ADON = 1;	//Start AN3 A/D converter.
	__delay_us(50);

	ADCON0bits.GO = 1;
	while(ADCON0bits.GO == 1);

	wADVal = ADRES;

	ADCON0bits.ADON = 0;	//End AN3 A/D converter.

	BIAS_PORT = LOW;

	return wADVal;
}

////////////////////////////////////////////////////////////////////////////
void beep(BYTE t)
{
	if(t == 0){
		T2CONbits.TMR2ON = 0;
	}
	else{
		PR2 = t;
		CCPR1L = t / 2;
		T2CONbits.TMR2ON = 1;
	}

}

////////////////////////////////////////////////////////////////////////////
void interrupt_onoff(BYTE bFlag)
{
	IOCAPbits.IOCAP3 = bFlag;
	INTCONbits.IOCIE = bFlag;
	INTCONbits.PEIE = bFlag;
}

////////////////////////////////////////////////////////////////////////////
bit get_sw()
{
	if(SW_PORT == LOW){		//DOWN
		__delay_ms(10);
		if(SW_PORT == LOW){
			for(;;){
				if(SW_PORT == HIGH){
					break;
				}
			}
			return 1;
		}
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////
void chip_initialize(void)
{
	OSCCON = 0b00101010;	//NbN125kHz.

	OPTION_REGbits.nWPUEN = 1;
							//vAbv.
	WPUA = 0b00000000;		//vAbv|[gݒ.

	TRISA = 0b00011000;		//0:o 1:.

	CM1CON0 = 0b00000000;	//Rp[^.

	ANSELA = 0b00010000;	//RA4AiO͂ɂ.

	FVRCON = 0b00000001;	//A/Dϊp̓d1.024V.

	ADCON0 = 0b00001100;	//AN3ADϊsɑIĂ.
	ADCON1 = 0b10000011;	//ADϊʂEl߁AdFVRɂ.

	APFCONbits.CCP1SEL = 0;	//PWMo͂RA2sɎw肷.
	CCP1CON = 0b00001100;	//PWML.

	T2CON = 0b00000000;		//Timer2̃vXP[1:1.

	LATA = 0b00000000;		//PORTA.

	interrupt_onoff(1);

	LED_PORT = 0;
}

////////////////////////////////////////////////////////////////////////////
void main(void)
{
	BYTE bLvd;
	WORD wSdt;
	WORD wADVal;
	static const BYTE bTone[6] = {
		7,
		7,
		11,
		17,
		27,
		40
	};

	chip_initialize();

	__delay_ms(100);

	while(1){
		BIAS_PORT = LOW;
		LED_PORT = 0;
		FVRCONbits.FVREN = 0;
		IOCAF = 0;

		interrupt_onoff(1);

		SLEEP();
		NOP();
		IOCAF = 0;
		__delay_ms(50);

		interrupt_onoff(0);

		FVRCONbits.FVREN = 1;
		LED_PORT = 1;
		wSdt = 0;

		wADVal = get_adval();

		if(wADVal < 344){	//2.0VȂ̂ŋNȂ.
			continue;
		}
		if(wADVal < 412){	//2.4VLED_łŒʒm.
			bLvd = 1;
		}
		else{				//ʏ̋N.
			bLvd = 0;
		}

		beep(bTone[0]);
		__delay_ms(100);
		beep(0);

		do{
			wADVal = get_adval();

			__delay_us(7200);

			if(wADVal < 6){
				beep(bTone[wADVal]);
				wSdt = 0;
			}
			else{
				beep(0);
			}
			if(bLvd && ++bLvd > 20){
				bLvd = 1;
				LED_PORT = !LED_PORT;
			}

			if(get_sw() == 1){
				break;
			}
		}while(++wSdt < 30000);	//5.
		//}while(++wSdt < 6000);	//1.
		//}while(++wSdt < 500);	//5b.

		beep(bTone[0]);
		__delay_ms(500);
		beep(0);
	}
}
